/** ***************************************************************************
 * @file  MemoryMappedFile.h
 * @brief MemoryMappedFileNX̎
 * @author Hidekazu Ito
 * @date 2015/05/10
 ******************************************************************************/

#include "MemoryMappedFile.h"

using Win32Utility::MemoryMappedFile;

/** *****************************************************************
 * @brief RXgN^
 ********************************************************************/
MemoryMappedFile::MemoryMappedFile()
: m_hMapFile(NULL)
, m_MapAddress(NULL)
, m_Size(0u)
{
}

/** *****************************************************************
 * @brief fXgN^
 ********************************************************************/
MemoryMappedFile::~MemoryMappedFile()
{
    Close();
}

/** *****************************************************************
 * @brief }bv̈m
 * @param[in]  fileSize  t@CTCY
 * @param[in]  fileName  }bvht@C(NULLw\)
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL MemoryMappedFile::Create(DWORD fileSize, LPCTSTR fileName)
{
    HANDLE hMapFile;
    LPVOID lpMapAddress;
    // t@C}bsOIuWFNg쐬
    hMapFile = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, fileSize, fileName);
    if (NULL == hMapFile) {
        return FALSE;
    }
    // }bsO
    lpMapAddress = MapViewOfFile(hMapFile, FILE_MAP_ALL_ACCESS, 0, 0, 0);
    if (NULL == lpMapAddress) {
        CloseHandle(hMapFile);
        return FALSE;
    }
    Close();
    m_hMapFile   = hMapFile;
    m_MapAddress = lpMapAddress;
    m_Size       = fileSize;
    return TRUE;
}

/** *****************************************************************
 * @brief }bvI[v
 * @param[in]  fileName  }bvht@C
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL MemoryMappedFile::Open(LPCTSTR fileName)
{
    HANDLE hMapFile;
    LPVOID lpMapAddress;
    MEMORY_BASIC_INFORMATION mbi = {};
    // t@C}bsOIuWFNgJ
    hMapFile = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, fileName);
    if (NULL == hMapFile) {
        return FALSE;
    }
    // }bsO
    lpMapAddress = MapViewOfFile(hMapFile, FILE_MAP_ALL_ACCESS, 0, 0, 0);
    if (NULL == lpMapAddress) {
        CloseHandle(hMapFile);
        return FALSE;
    }
    if (sizeof(mbi) != VirtualQuery(m_MapAddress, &mbi, sizeof(mbi))) {
        CloseHandle(hMapFile);
        return FALSE;
    }
    Close();
    m_hMapFile   = hMapFile;
    m_MapAddress = lpMapAddress;
    m_Size       = mbi.RegionSize;
    return TRUE;
}

/** *****************************************************************
 * @brief }bvN[Y
 ********************************************************************/
VOID MemoryMappedFile::Close(VOID)
{
    if (NULL != m_MapAddress) {
        Flush();
        // }bsO
        UnmapViewOfFile(m_MapAddress);
        m_MapAddress = NULL;
    }
    // t@C}bsOIuWFNg
    if (NULL != m_hMapFile) {
        CloseHandle(m_hMapFile);
        m_hMapFile = NULL;
    }
    m_Size = 0u;
}

/** *****************************************************************
 * @brief }bvAhX擾
 * @return  }bvAhXԂ
 ********************************************************************/
LPVOID MemoryMappedFile::GetMappedAddress(VOID) CONST
{
    return m_MapAddress;
}

/** *****************************************************************
 * @brief }bvTCY擾
 * @return  }bvTCYԂ
 ********************************************************************/
DWORD MemoryMappedFile::GetSize(VOID) CONST
{
    return m_Size;
}

/** *****************************************************************
 * @brief }bṽtbV(XV)
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL MemoryMappedFile::Flush(VOID)
{
    return FlushViewOfFile(m_MapAddress, 0);
}

/** *****************************************************************
 * @brief }bv֏
 * @param[in]  data      ݃f[^
 * @param[in]  dataSize  f[^TCY
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL MemoryMappedFile::Write(LPVOID data, DWORD dataSize)
{
    if (NULL == m_MapAddress) {
        return FALSE;
    }
    CopyMemory(m_MapAddress, data, dataSize);
    return TRUE;
}

/** *****************************************************************
 * @brief }bvǂݍ
 * @param[in]  buffer      ݃f[^
 * @param[in]  bufferSize  f[^TCY
 * @retval  FALSE  s
 * @retval  TRUE   
 ********************************************************************/
BOOL MemoryMappedFile::Read(LPVOID buffer, DWORD bufferSize, LPDWORD readSize)
{
    if (NULL == m_MapAddress) {
        return FALSE;
    }
    CopyMemory(buffer, m_MapAddress, bufferSize);
    if (NULL != readSize) {
        *readSize = bufferSize;
    }
    return TRUE;
}

// EOF